/** @module redux-gen/basic */

/**
  * Generates everything which is usually required to integrate a simple module into
  * a React-Redux environment and puts it in an object for easier integration.
  *
  * Usage:
  * Suppose you want to integrate your module 'user' in the React-Redux universe.
  * call `const user = reduxGen('user')`
  * user now contains
  *   - the action types `CREATE`, `UPDATE` and `DELETE`
  *   - the actions `createUser`, `updateUser` and `deleteUser`
  *   - the state subpart `{ user: [] }`
  *   - the reducer which handles the actions in a default manner.
  *
  * Integrate the user as follows into your project:
  *   - Extend your default state
  *     `const defaultState = { <some definitions>, ...user.state }`
  *   - Create the root reducer
  *     `const rootReducer = combineReducers({ <some definitions>, ...user.reducer })`
  *   - Create the Redux store
  *     `const store = createStore(rootReducer, defaultState, <enhancers>)`
  *   - Bind the actions to the dispatcher
  *   - Connect your app to the store
  *
  * @param {string} name - Name of the module
  */
export default function reduxGen (name) {
  // Start with empty objects
  const actionTypes = {}
  const actions = {}

  // Define the default actions to be handled by reduxGen
  const actionList = ['create', 'update', 'delete']

  /** Populate the actionTypes and actions */
  actionList.forEach(action => {
    // actionType is sth like CREATE_ITEM
    const actionType = `${action.toUpperCase()}_${name.toUpperCase()}`
    // actionName is sth like createItem
    const actionName = `${action}${name[0].toUpperCase()}${name.substring(1)}`

    // Add the action types and actions to the array
    actionTypes[actionType] = `${name}/${actionType}`
    actions[actionName] = (id, data) => { return { type: `${name}/${actionType}`, id, data } }
  })

  // Default state.
  const state = []

  // Reducer
  function reducer (state = [], action) {
    if (!action) {
      return state
    }
    let nextState
    switch (action.type) {
      case actionTypes[`CREATE_${name.toUpperCase()}`]:
        nextState = [ ...state, action.data ]
        return nextState
      case actionTypes[`UPDATE_${name.toUpperCase()}`]:
        nextState = [ ...state.slice(0, action.id), { ...state[action.id], ...action.data }, ...state.slice(action.id + 1) ]
        return nextState
      case actionTypes[`REMOVE_${name.toUpperCase()}`]:
        nextState = [ ...state.slice(0, action.id), ...state.slice(action.id + 1) ]
        return nextState
      default:
        return state
    }
  }

  return {
    actionTypes: { [name]: actionTypes },
    actions: { [name]: actions },
    state: { [name]: state },
    reducer: { [name]: reducer }
  }
}
